/* pmatrix.h: by Jonah Cohen
 *
 * This is a matrix class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apmatrix class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pmatrix.h is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2000
 */

#ifndef _PMATRIX_H_
#define _PMATRIX_H_

#include "pvector.h"

template <class T>
class pmatrix
{

public:
	pmatrix();											//default constructor
	pmatrix(int rows, int cols);						//constructor with dimensions
	pmatrix(int rows, int cols, const T & fillvalue);	//constructor with default fill value
	pmatrix(const pmatrix<T> &);						//copy constructor
	virtual ~pmatrix();									//destructor

	void resize(int rows, int cols);					//change matrix dimensions
	inline int numrows() const;							//returns number of rows in pmatrix
	inline int numcols() const;							//returns number of columns in pmatrix

	inline pvector<T> & operator [] (int index);			//access a particular array element (mutable)
	inline const pvector<T> & operator [] (int index) const;//access a particular array element (immutable)

	const pmatrix<T> & operator = (const pmatrix<T> &);		//assignment operator

protected:
	pvector< pvector<T> > matrix;

};

#include"pmatrix.cpp"

#endif
