/* pqueue.cpp: by Jonah Cohen
 *
 * This is a queue class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apqueue class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pqueue.cpp is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#include "pqueue.h"
#include <iostream>

template <class T>
pqueue<T>::pqueue() :head(0), tail(0), size(0)
{}

template <class T>
pqueue<T>::pqueue(const pqueue<T> &copy) :head(0), tail(0), size(0)
{
	*this = copy;
}

template <class T>
pqueue<T>::~pqueue()
{
	makeEmpty();
}

template <class T>
void pqueue<T>::enqueue(const T &data)
{
	node *newnode = new node(data);	//end of queue
	
	if(size==0)	//make sure queue exists
	{
		head = newnode;
		tail = head;
	}
	else
	{
		tail->next = newnode;
		tail = newnode;
	}
	++size;
}

template <class T>
void pqueue<T>::dequeue(T &storage)
{
	if(head==0)
	{
		cerr << "\nError: accessing empty queue through method pqueue::dequeue\n";
		exit(1);
	}

	storage = head->data;				//save to var
	node *temp = head;					//make a new node
	head = head->next;					//iterate
	delete temp;						//delete the node
	--size;								//decrement size
}

template <class T>
const T pqueue<T>::dequeue()
{
	T val;				//store to temporary variable
	dequeue(val);		//dequeue
	return val;
}

template <class T>
const T & pqueue<T>::front() const
{
	if(head==0)
	{
		cerr << "\nError: accessing empty queue through method pqueue::front\n";
		exit(1);
	}
	return head->data;
}

template <class T>
void pqueue<T>::makeEmpty()
{
	node *temp;
	while(head)
	{
		temp = head;
		head = head->next;
		delete temp;
	}
	size = 0;
}

template <class T>
const pqueue<T> & pqueue<T>::operator = (const pqueue<T> & copy)
{
	makeEmpty();
	if(copy.isEmpty())
		return *this;

	head = new node(copy.head->data);
	tail = head;
	node *newnode;
	for(newnode = copy.head->next; newnode; newnode = newnode->next)
	{
		tail->next = new node(newnode->data);
		tail = tail->next;
	}
	size = copy.size;
	return *this;
}

template <class T> 
int pqueue<T>::length() const
{
	return size;
}

template <class T> 
bool pqueue<T>::isEmpty() const
{
	return size == 0;
}

