/* pqueue.h: by Jonah Cohen
 *
 * This is a queue class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apqueue class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pqueue.h is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2000
 */

#ifndef _PQUEUE_H_
#define _PQUEUE_H_

template <class T>
class pqueue
{

public:
	pqueue();								//default constructor
	pqueue(const pqueue<T> &);				//copy constructor
	virtual ~pqueue();						//destructor

	void enqueue(const T &data);			//enqueue data to queue
	void dequeue(T &storage);				//storage holds the data that is dequeued
	const T dequeue();						//dequeue to return value
	const T & front() const;				//returns top value without dequeueing

	void makeEmpty();						//empty the queue

	inline bool isEmpty() const;			//returns true if queue is empty
	inline int length() const;				//find out the number of queued items

	const pqueue<T> & operator = (const pqueue<T> &);	//assignment operator

protected:
	struct node
	{
		T data;
		node *next;
		node() :next(0) {}
		node(const T &a) :next(0), data(a) {}
	} *head, *tail;
	int size;

};

#include"pqueue.cpp"

#endif

