/* pstack.cpp: by Jonah Cohen
 *  
 * This is a stack class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apstack class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pstack.cpp is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#include "pstack.h"
#include <iostream>

template <class T>
pstack<T>::pstack() :sp(0), size(0)
{}

template <class T>
pstack<T>::pstack(const pstack<T> &copy) :sp(0), size(0)
{
	*this = copy;
}

template <class T>
pstack<T>::~pstack()
{
	makeEmpty();
}

//push a value onto the stack
template <class T>
void pstack<T>::push(T storage)
{
	//create new data
	node *newnode = new node;

	newnode->next=sp;
	newnode->data=storage;
	sp = newnode;
	++size;
}

//pop a value off of stack
template <class T>
void pstack<T>::pop(T & storage)
{
	if(isEmpty())
	{
		cerr << "\nError: accessing empty stack through method pstack::pop\n";
		exit(1);
	}

	//store data
	storage = sp->data;

	//delete node and move stack pointer back one
	node *temp = sp;
	sp = sp->next;
	delete temp;
	--size;
}

//pop a value off of stack and return it
template <class T>
const T pstack<T>::pop()
{
	T val;
	pop(val);
	return val;
}

template <class T>
const T pstack<T>::top() const
{
	if(isEmpty())
	{
		cerr << "\nError: accessing empty stack through method pstack::top\n";
		exit(1);
	}

	return sp->data;
}

template <class T>
int pstack<T>::length() const
{
	return size;
}

template <class T>
void pstack<T>::makeEmpty()
{
	node *temp;
	//iterate through list, deleting each elements
	while(sp!=0)
	{
		temp = sp;
		sp = sp->next;
		delete temp;
	}
	size = 0;
}

template <class T>
bool pstack<T>::isEmpty() const
{
	return size == 0;
}

template <class T>
const pstack<T> & pstack<T>::operator = (const pstack<T> & copy)
{
	makeEmpty();
	if(copy.isEmpty())
		return *this;

	sp = new node(copy.sp->data);
	node *newnode,*end = sp;
	for(newnode = copy.sp->next; newnode; newnode = newnode->next)
	{
		end->next = new node(newnode->data);
		end = end->next;
	}
	size = copy.size;
	return *this;
}
