/* pstack.h: by Jonah Cohen
 *
 * This is a stack class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apstack class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pstack.h is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#ifndef _PSTACK_H_
#define _PSTACK_H_

template <class T>
class pstack
{

public:
	pstack();									//default constructor
	pstack(const pstack<T> &);					//copy constructor
	virtual ~pstack();							//destructor

	void push(T storage);						//push data onto stack
	void pop(T & storage);						//pop data off stack and store in parameter
	const T pop();								//pop data off stack and return
	const T top() const;						//returns top value without popping

	void makeEmpty();							//empty the stack

	inline bool isEmpty() const;				//returns true if stack is empty
	inline int length() const;					//returns number of elements on stack

	const pstack<T> & operator = (const pstack<T> &);	//assignment operator

protected:
	struct node
	{
		T data;
		node *next;
		node() :next(0) {}
		node(const T & a) :next(0), data(a) {}
	} *sp;
	int size;

};

#include "pstack.cpp"

#endif

