/* pstring.cpp: by Jonah Cohen
 *
 * This is a string class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apstring class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pvector.cpp is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#include "pstring.h"
#include <stdio.h>
#include <string>

ostream & operator << (ostream &os, const pstring &out)
{
	return os << out.c_str() << flush;
}

istream & operator >> (istream &is, pstring &in)
{
	fflush(stdin);
	char input_buffer[4096];
	is >> input_buffer;
	in = input_buffer;
	return is;
}

istream & getline(istream &is, pstring &to_get)
{
	fflush(stdin);
	char getline_buffer[4096];
	is.getline(getline_buffer, sizeof(getline_buffer)-1, '\n');
	to_get = getline_buffer;
	return is;
}

pstring::pstring()
{
	mystring = new char[1];
	mystring[0] = '\0';
}

pstring::pstring(const char *copy)
{
	mystring = new char[strlen(copy) + 1];	//allocate memory
	strcpy(mystring, copy);					//copy string
}

pstring::pstring(char ch)
{
	mystring = new char[2];
	mystring[0] = ch;
	mystring[1] = '\0';
}

pstring::pstring(const pstring & to_create_from)
{
	mystring = new char[to_create_from.length()+1];
	strcpy(mystring, to_create_from.c_str());	//copy string
}

pstring::~pstring()
{
	delete[] mystring;
}

char* pstring::c_str() const
{
	return mystring;
}

int pstring::length() const
{
	return strlen(mystring);
}

int pstring::find(const pstring & str) const
{
	int i, j, endsearch = length() - str.length() + 1;
	for(i = 0; i < endsearch; i++)
	{
		for(; i < endsearch && mystring[i] != str[0]; i++);
		if(i == endsearch)
			break;

		for(j = 0; j < str.length() && mystring[i+j] == str[j]; j++);
		if(j == str.length())
				return i;
	}
	return -1;
}

int pstring::find(const char ch) const 
{
	for(int i = 0; i < length(); i++)
		if(mystring[i] == ch)
			return i;

	return -1;
}

pstring pstring::substr(int pos, int len) const
{
	if(pos < 0 || len < 0 || pos >= length())
	{
		cerr << "\nError: substring (" << pos << "," << len
			 << ") out of bounds for string " << mystring << endl;
		exit(1);
	}
	if(pos + len > length())
		len = length() - pos;
	char *result = new char[len + 1];
	memcpy(result, mystring + pos, len);
	result[len] = '\0';
	pstring to_return(result);
	delete[] result;
	return to_return;
}

const pstring & pstring::operator =(const char * const to_copy)
{
	char *newstring = new char[strlen(to_copy)+1];
	strcpy(newstring, to_copy);			//copy string
	delete[] mystring;					//deallocate mem
	mystring = newstring;				//points to new string
	return *this;
}

const pstring & pstring::operator =(const char ch)
{
	delete[] mystring;					//deallocate memory
	mystring = new char[2];
	mystring[0] = ch;
	mystring[1] = '\0';
	return *this;
}

const pstring & pstring::operator =(const pstring &copy)
{
	return *this = copy.c_str();		//call char pointer copier
}

const pstring & pstring::operator +=(const char * const to_append)
{
	char *newbuffer = new char[length() + strlen(to_append) + 1];
	strcpy(newbuffer, mystring);
	strcat(newbuffer, to_append);
	delete[] mystring;
	mystring = newbuffer;
	return *this;
}

const pstring & pstring::operator +=(const pstring &to_append)
{
	return *this += to_append.c_str();	//append char pointer
}

const pstring & pstring::operator +=(const char to_append)
{
	char *newstring = new char[length()+2];
	strcpy(newstring,mystring);
	delete[] mystring;
	mystring = newstring;				//points to new string
	mystring[length()+1] = '\0';		//null terminator
	mystring[length()] = to_append;
	return *this;
}

pstring operator +(const pstring & lval, const pstring & rval)
{
	pstring to_return(lval);
	to_return += rval;
	return to_return;
}

pstring operator +(const pstring & lval, const char & ch)
{
	pstring to_return(lval);
	to_return += ch;
	return to_return;
}

pstring operator +(char ch, const pstring & str)
{
	pstring to_return(ch);
	to_return += str;
	return to_return;
}

char & pstring::operator[](int n)
{
	if(n<0 || n>=length())
	{
		cerr << "\nError: index out of range: " << n << " in string \""
			 << mystring << "\" of length " << length() << endl;
		exit(1);
	}
	return mystring[n];
}

const char pstring::operator[](int n) const
{
	if(n<0 || n>=length())
	{
		cerr << "\nError: index out of range: " << n << " in string \""
			 << mystring << "\" of length " << length() << endl;
		exit(1);
	}
	return mystring[n];
}

bool operator ==(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(), rval.c_str()) == 0;
}

bool operator !=(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(),rval.c_str()) != 0;
}

bool operator <(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(), rval.c_str()) < 0;
}

bool operator <=(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(), rval.c_str()) <= 0;
}

bool operator >(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(), rval.c_str()) > 0;
}

bool operator >=(const pstring &lval, const pstring &rval)
{
	return strcmp(lval.c_str(), rval.c_str()) >= 0;
}
