/* pstring.h: by Jonah Cohen
 *
 * This is a string class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apstring class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pstring.h is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#ifndef _PSTRING_H_
#define _PSTRING_H_

#include <iostream>

class pstring
{

public:
	pstring(); 									//default constructor
	pstring(const pstring &);					//copy constructor
	pstring(const char *copy);					//copy constructor from C-style string
	pstring(char ch);							//copy constructor from single char
	virtual ~pstring();							//destructor

	inline char * c_str() const;				//returns a null-terminated, C-style string
	inline int length() const;					//returns the number of characters in the string

	int find(const pstring &str) const;			//return index of str or -1 if not found
	int find(const char ch) const;				//return index of ch or -1 if not found
	pstring substr(int pos, int len) const;		//returns substring from pos of length len
	
	char & operator [] (int n);					//access a character (mutable)
	const char operator [] (int n) const;		//access a character (immutable)

	const pstring & operator = (const pstring &);		//assignment operator
	const pstring & operator = (const char * const);	//assignment operator from C-style string
	const pstring & operator = (const char);			//assignment operator from single char

	const pstring & operator += (const pstring &);		//concatenation operator
	const pstring & operator += (const char * const);	//concatenation operator from C-style string
	const pstring & operator += (const char);			//concatenation operator from single char

protected:
	char *mystring;

};

//concatenation operators

pstring operator + (const pstring &, char);
pstring operator + (const pstring &, const pstring &);
pstring operator + (char ch, const pstring &);

//stream operators

inline ostream & operator << (ostream &, const pstring &);
istream & operator >> (istream &, pstring &);
istream & getline(istream &, pstring &);

//comparison operators

inline bool operator == (const pstring &, const pstring &);
inline bool operator != (const pstring &, const pstring &);
inline bool operator <  (const pstring &, const pstring &);
inline bool operator <= (const pstring &, const pstring &);
inline bool operator >  (const pstring &, const pstring &);
inline bool operator >= (const pstring &, const pstring &);

#endif

