/* pvector.cpp: by Jonah Cohen
 *  
 * This is a vector class designed to work without any
 * jury-rigging, crying, sweat, blood, or tears when substituted for
 * the apvector class published by the College Board for use in the
 * AP Computer Science curriculum.
 *
 * pvector.cpp is distributed under the GNU Public License agreement (GPL)
 * for distribution, use, sale, modification, and obfuscation at NO CHARGE.
 * However, any use of this code must be accompanied by a thorough and
 * conspicuous acknowledgement of the use of open source code, as well as of
 * the original author or authors.
 *
 * Last modified March 23, 2001
 */

#include "pvector.h"
#include <iostream>

template <class T>
pvector<T>::pvector() :array(0), len(0)
{}

template <class T>
pvector<T>::pvector(int size)
{
	if(size <= 0)
	{
		cerr << "\nError: invalid pvector dimension: " << size << endl;
		exit(1);
	}
	array = new T[size];
	len = size;
}

template <class T>
pvector<T>::pvector(int size, const T &fill_val)
{
	array = new T[size];
	len = size;

	for(int i=0; i<size; i++)
		array[i] = fill_val;
}

template <class T>
pvector<T>::pvector(const pvector<T> &vec)
{
	array = new T[vec.length()];
	for(int i=0; i<vec.length(); i++)
		array[i] = vec[i];
	len = vec.length();
}

template <class T>
pvector<T>::~pvector()
{
	delete[] array;
}

template <class T>
int pvector<T>::length() const
{
	return len;
}

template <class T>
T & pvector<T>::operator [] (int index)
{
	if(index < 0 || index >= length())
	{
		cerr << "\nError: index out of range: " << index
			 << " in pvector of length " << length() << endl;
		exit(1);
	}
	return array[index];
}

template <class T>
const T & pvector<T>::operator [] (int index) const
{
	if(index < 0 || index >= length())
	{
		cerr << "\nError: index out of range: " << index
			 << " in pvector of length " << length() << endl;
		exit(1);
	}
	return array[index];
}

template <class T>
const pvector<T> & pvector<T>::operator = (const pvector<T> & vec)
{
	delete[] array;
	array = new T[vec.length()];
	for(int i=0; i<vec.length(); i++)
		array[i] = vec[i];
	len = vec.length();
	return *this;
}

template <class T>
void pvector<T>::resize(int new_size)
{
	if(new_size <= 0)
	{
		cerr << "\nError: invalid pvector dimension: " << new_size << endl;
		exit(1);
	}

	T *newarray = new T[new_size];

	int minsize = (new_size<len)?new_size:len;
	for(int i=0; i<minsize; i++)
		newarray[i] = array[i];

	delete[] array;
	array = newarray;
	len = new_size;
}
